'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _passport = require('passport');

var _passport2 = _interopRequireDefault(_passport);

var _jsonwebtoken = require('jsonwebtoken');

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// http://www.iana.org/assignments/jwt/jwt.xhtml
var defaultMapUserToJWTProps = function defaultMapUserToJWTProps(user) {
  return { userId: user.id };
};
var defaultMapUserToUserId = function defaultMapUserToUserId(user) {
  return user.id;
};

function defaultCreateTokenFromUser(user) {
  return _jsonwebtoken2.default.sign(this.mapUserToJWTProps(user), this.jwtSecret, { expiresIn: "7 days" });
}

var ApolloPassport = function () {
  function ApolloPassport() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, ApolloPassport);

    // TODO actually let's rather default to a DB request on each query if no jwt secret specified
    var jwtSecret = options.jwtSecret || process.env.JWT_SECRET;
    if (!jwtSecret && !options.createTokenFromUser) throw new Error("no jwtSecret nor a custom createTokenFromUser specified");

    this._useJwt = true;

    this._strategies = new Map();
    this._winston = options.winston || require('winston');

    this.dbName = options.db[0];
    var userTableName = options.userTableName || 'users';

    var DBDriver = require('./db/' + this.dbName).default;
    this.db = new DBDriver(options.db[1], userTableName);

    this.jwtSecret = jwtSecret;
    this.mapUserToJWTProps = options.mapUserToJWTProps || defaultMapUserToJWTProps;
    this.createTokenFromUser = options.createTokenFromUser || defaultCreateTokenFromUser;
    this.mapUserToUserId = options.mapUserToUserId || defaultMapUserToUserId;
  }

  _createClass(ApolloPassport, [{
    key: 'use',
    value: function use(name, Strategy, options, verify) {
      if (!verify) {
        verify = options;
        options = null;
      }

      if (!options) options = this.require(name, 'defaultOptions');
      if (!verify) verify = this.dbRequire(name, 'verify');

      var instance = new Strategy(options, verify.bind(this));
      _passport2.default.use(instance);

      //this._strategies.set(instance.name, instance);
    }
  }, {
    key: 'require',
    value: function (_require) {
      function require(_x, _x2) {
        return _require.apply(this, arguments);
      }

      require.toString = function () {
        return _require.toString();
      };

      return require;
    }(function (strategy, module) {
      // No static analysis, but that's ok for server side
      var loaded = require('./strategies/' + strategy + '/' + module);
      return loaded.default;
    })
  }, {
    key: 'dbRequire',
    value: function dbRequire(strategy, module) {
      return this.require(strategy, 'db/' + this.dbName + '/' + module);
    }
  }, {
    key: 'schema',
    value: function schema() {
      return require('./schema').default;
    }
  }, {
    key: 'resolvers',
    value: function resolvers() {
      return this._bindRootMutations(require('./resolvers').default);
    }
  }, {
    key: 'expressMiddleware',
    value: function expressMiddleware() {
      var path = arguments.length <= 0 || arguments[0] === undefined ? '/ap-auth' : arguments[0];

      return function ApolloPassportExpressMiddleware(req, res, next) {
        console.log('!!!', req.url);
      };
    }
  }, {
    key: 'wrapOptions',
    value: function wrapOptions(options) {
      var self = this;

      return function _callee(req) {
        var token, decoded;
        return regeneratorRuntime.async(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (req.headers.authorization) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', options);

              case 2:
                token = req.headers.authorization.substr(0, 7) === 'Bearer ' && req.headers.authorization.substr(7);

                if (token) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt('return', options);

              case 5:

                console.log('token', token);

                decoded = void 0;
                _context.prev = 7;

                decoded = _jsonwebtoken2.default.verify(token, self.jwtSecret);
                console.log('decoded', decoded);
                _context.next = 16;
                break;

              case 12:
                _context.prev = 12;
                _context.t0 = _context['catch'](7);

                // This could be any number of reasons but in short: user not authed.
                // Maybe pass the error to context so we can send back to client.
                // JsonWebTokenError: invalid token
                // TokenExpiredError: jwt expired
                console.log('err', _context.t0);
                return _context.abrupt('return', options);

              case 16:
                return _context.abrupt('return', _extends({}, options, {
                  context: { jwt: decoded }
                }));

              case 17:
              case 'end':
                return _context.stop();
            }
          }
        }, null, this, [[7, 12]]);
      };
    }
  }, {
    key: '_bindRootMutations',
    value: function _bindRootMutations(obj) {
      var out = {};
      for (var key in obj) {
        if (key === 'RootMutation') {
          out.RootMutation = {};
          for (var key2 in obj.RootMutation) {
            out.RootMutation[key2] = obj.RootMutation[key2].bind(this);
          }
        } else {
          out[key] = obj[key];
        }
      }return out;
    }
  }]);

  return ApolloPassport;
}();

exports.default = ApolloPassport;