"use strict";

var _proxyquire = require("proxyquire");

var _proxyquire2 = _interopRequireDefault(_proxyquire);

var _chai = require("chai");

var _chai2 = _interopRequireDefault(_chai);

var _chaiAsPromised = require("chai-as-promised");

var _chaiAsPromised2 = _interopRequireDefault(_chaiAsPromised);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use(_chaiAsPromised2.default);
_chai2.default.should();

// might be useful elsewhere too...
var passportStub = {
  authenticate: function authenticate(name, callback) {
    return function (req) {
      if (!req) throw new Error("no req");
      if (!req.query) throw new Error("no req.query");

      var override = req.query.override;
      if (!override) throw new Error("no req.query.override");

      switch (override) {
        case 'error':
          return callback(new Error());
        case 'user':
          return callback(null, { id: 1 });
        case 'info':
          return callback(null, null, 'info');
        default:
          throw new Error("Invalid req.query.override: " + override);
      }
    };
  }
};

var context = {
  mapUserToUserId: function mapUserToUserId(user) {
    return user.id;
  },
  createTokenFromUser: function createTokenFromUser() {
    return 'token';
  }
};

var resolvers = (0, _proxyquire2.default)('./resolvers', { passport: passportStub }).default;

describe('resolvers', function () {

  describe('local', function () {

    describe('passportLoginEmail', function () {

      var passportLoginEmail = resolvers.RootMutation.passportLoginEmail.bind(context);

      it('throws on a real error', function () {
        // I don't know how to check if an async function throws
        passportLoginEmail(null, { override: 'error' }).should.be.rejected;
      });

      it('passes the user', function _callee() {
        var result;
        return regeneratorRuntime.async(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return regeneratorRuntime.awrap(passportLoginEmail(null, { override: 'user' }));

              case 2:
                result = _context.sent;

                result.error.should.equal("");
                result.token.should.equal('token');
                result.userId.should.equal(1);

              case 6:
              case "end":
                return _context.stop();
            }
          }
        }, null, undefined);
      });

      it('passes an error string (not a throw)', function _callee2() {
        var result;
        return regeneratorRuntime.async(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return regeneratorRuntime.awrap(passportLoginEmail(null, { override: 'info' }));

              case 2:
                result = _context2.sent;

                result.error.should.equal("info");
                result.token.should.equal("");
                result.userId.should.equal("");

              case 6:
              case "end":
                return _context2.stop();
            }
          }
        }, null, undefined);
      });
    });
  });
});